module.exports = function dragDropSave(request, root) {
    // Добавляем всем button и a элементам дата-атрибут draggable для визуализации эффекта
    root.childNodes?.forEach(node => node.childNodes.forEach(itm => {
        itm.setAttribute('draggable', 'true')
    }))

    // Функция для смены позиций двух элементов внутри JSON файла
    const swapItemsInJson = (blockNumber, firstItem, secondItem) => {
        const block = request.structure[blockNumber].children;
        [block[firstItem], block[secondItem]] = [block[secondItem], block[firstItem]]
    }

    // Функция для замены одного dom элемента на другой
    const swap = (activeItem, currentItem) => {
        let afterNode2 = currentItem.nextElementSibling === activeItem
            ? currentItem
            : currentItem.nextElementSibling
        const parent = currentItem.parentNode
        activeItem.replaceWith(currentItem)
        parent.insertBefore(activeItem, afterNode2)
    }

    // Перебираем все корневые div, для добавления к ним слушателей событий
    root.childNodes?.forEach((node, nodeIndex) => {

        // Перемещаемый элемент
        let activeElement
        // Элемент на котором находится курсор
        let currentElement

        // Присваиваем дата атрибут для всех внутренних элементов, для их уникальной идентификации
        const setDataAtr = () => {
            node.querySelectorAll('*')
                .forEach((insideElem, idx) =>
                    insideElem.dataset.index = idx.toString())
        }
        setDataAtr()

        node.addEventListener(`dragstart`, e => {
            e.target.classList.add('selected')
        })

        node.addEventListener(`dragover`, e => {
            e.preventDefault()

            activeElement = node.querySelector('.selected')

            currentElement = e.target
        })

        node.addEventListener(`dragend`, e => {
            e.target.classList.remove(`selected`)

            if (activeElement === currentElement) {
                return
            }

            const cantSwap = activeElement.tagName === currentElement.tagName

            if (cantSwap) {
                alert('Поменять местами можно только разные элементы. Например button и link')
            } else if (!currentElement.draggable) {
                alert('Перемещение элементов доступно только внутри блоков')
            } else if (activeElement.draggable && currentElement.draggable && activeElement) {
                swap(activeElement, currentElement)
                swapItemsInJson(nodeIndex, +currentElement.dataset.index, +activeElement.dataset.index)
                setDataAtr()
            }
        })
    })

    // Работа с кнопкой сохранения
    const saveBtn = document.querySelector('[data-save]')
    saveBtn.addEventListener('click', () => {
        console.log('JSON Structure:', JSON.stringify(request.structure))
        console.log('JSON Structure:', request.structure)
    })
}